﻿#Region Init
Clear-Host


[string]$CurrentDir = (Get-Location).path
[string]$RepoPath   = "$CurrentDir\repo"
[string]$fossil_exe = "$CurrentDir\fossil.exe"


#EndRegion

#region Functions

Function New-Repository {
	[CmdletBinding()]
	[OutputType([hashtable])]
	Param(
		[parameter(Mandatory=$true)]
		[alias("Key")]
		[string]$Name, 
		[parameter(Mandatory=$true)]
		[string]$Description,
		[parameter(Mandatory=$true)]
		[string]$URLPath="" ,
		[parameter(Mandatory=$false)]
		[string]$Path="",
		[parameter(Mandatory=$false)]
		[string]$RepoPath="" 
	)
		
	if ($Path -eq "")     { $Path     = "$($script:CurrentDir)\$name" }
	if ($RepoPath -eq "") { $RepoPath = "$($script:RepoPath)\$name.fossil" }
	
	Write-Output @{
		Name        = $Name
		Description = $Description
		Path		= $Path
		URLPath     = $URLPath
		RepoPath    = $RepoPath
		}
}

Function Run-Fossil {
	[CmdletBinding()]
	[OutputType([boolean])]
	Param(
		[parameter(Mandatory=$true)]
		[string]$Command,
		[parameter(Mandatory=$false)]
		[string[]]$Arguments=@(),
		[parameter(Mandatory=$false)]
		[string]$Path = $script:CurrentDir
	)

	[string[]]$ArgumentList = @($Command) + $Arguments
	
	Write-Debug "$($ArgumentList)"

	Set-Location -Path $Path
	$process = Start-Process -FilePath $script:fossil_exe -ArgumentList $ArgumentList -Wait -NoNewWindow -PassThru
	Set-Location -Path $script:CurrentDir
	
	Write-Output ($process.ExitCode -eq 0)
}

Function New-Folder {
	[CmdletBinding()]
	[OutputType([void])]
	Param(
		[parameter(Mandatory=$true)]
		[string]$Path
	)

	If(-not (Test-Path -Path $Path)) { 
		New-Item -ItemType directory -Path $Path -ErrorAction Stop | Out-Null 
	}
}

Function Test-Repository {
	[CmdletBinding()]
	[OutputType([hashtable],[void])]
	Param(
		[parameter(Mandatory=$true,ValueFromPipeline=$true)]
		[hashtable]$Repository,
		[switch]$PassThru
	)

	process {
		New-Folder -Path $Repository.Path
		if($PassThru) { Write-Output $Repository }
	}
}

Function Clone-Repository {
	[CmdletBinding()]
	[OutputType([hashtable],[void])]
	Param(
		[parameter(Mandatory=$true,ValueFromPipeline=$true)]
		[hashtable]$Repository,
		[switch]$PassThru
	)
	
	process {
	
		If (-not (Test-Path -Path $Repository.RepoPath)) {

			Write-Host ""
			Write-Host "Клонирование" $Repository.Description
			Write-Host ""		
			
			[string[]]$Arguments = @(
				$Repository.URLPath, 
				$Repository.RepoPath
				)		
			[boolean]$ExitStatus = Run-Fossil -Command clone -Arguments $Arguments
			If ($ExitStatus -eq $false) { Throw "Не удалось клонировать $($Repository.Description)"	}
		}
	
		if($PassThru) { Write-Output $Repository }
	}
}

Function Open-Repository {
	[CmdletBinding()]
	[OutputType([hashtable],[void])]
	Param(
		[parameter(Mandatory=$true,ValueFromPipeline=$true)]
		[hashtable]$Repository,
		[switch]$PassThru
	)
	
	process {
		
		If (-not (Test-Path -Path "$($Repository.Path)\_fossil_")) {
			[boolean]$ExitStatus = Run-Fossil -Command open -Arguments $Repository.RepoPath -Path $Repository.Path
			If ($ExitStatus -eq $false) { Throw "Не удалось открыть $($Repository.Description)" }
		}

		if($PassThru) { Write-Output $Repository }
	}
}

Function Update-Repository {
	[CmdletBinding()]
	[OutputType([hashtable],[void])]
	Param(
		[parameter(Mandatory=$true,ValueFromPipeline=$true)]
		[hashtable]$Repository,
		[switch]$PassThru
	)
	
	process {
		Run-Fossil -Command pull -Path $Repository.Path | Out-Null
		Run-Fossil -Command update -Path $Repository.Path | Out-Null
		if($PassThru) { Write-Output $Repository }
	}
}

Function Close-Repository {
	[CmdletBinding()]
	[OutputType([hashtable],[void])]
	Param(
		[parameter(Mandatory=$true,ValueFromPipeline=$true)]
		[hashtable]$Repository,
		[switch]$PassThru
	)
	
	process {
	
		Run-Fossil -Command close -Path $Repository.Path | Out-Null
		
		if($PassThru) { Write-Output $Repository }
	}

	end {
	}
}

#EndRegion

New-Folder -Path $RepoPath

[hashtable[]]$Repositories = @(
	(New-Repository -Name "core" -URLPath "http://snegopat.ru:9004" -Description "репозиторий Снегопата 2.0" -Path $CurrentDir )
) 

$Repositories |
	Test-Repository	-PassThru 	| 
	Clone-Repository -PassThru | 
	Open-Repository -PassThru | 
	Update-Repository -PassThru | 
	Close-Repository